@@ -42,7 +42,7 @@ module Database.PostgreSQL.Simple
     -- ** Modifying multiple rows at once
     -- $many
 
-    -- ** @RETURNING@: modifications that returns results
+    -- ** @RETURNING@: modifications that return results
     -- $returning
 
     -- * Extracting results
@@ -93,6 +93,14 @@ module Database.PostgreSQL.Simple
     , forEach
     , forEach_
     , returning
+    -- ** Queries that stream results taking a parser as an argument
+    , foldWith
+    , foldWithOptionsAndParser
+    , foldWith_
+    , foldWithOptionsAndParser_
+    , forEachWith
+    , forEachWith_
+    , returningWith
     -- * Statements that do not return results
     , execute
     , execute_
@@ -110,18 +118,16 @@ module Database.PostgreSQL.Simple
     , formatQuery
     ) where
 
-import           Blaze.ByteString.Builder
-                   ( Builder, fromByteString, toByteString )
-import           Blaze.ByteString.Builder.Char8 (fromChar)
-import           Blaze.Text ( integral )
+import           Data.ByteString.Builder
+                   ( Builder, byteString, char8, intDec )
 import           Control.Applicative ((<$>))
 import           Control.Exception as E
-import           Control.Monad (foldM)
+import           Control.Monad (unless)
 import           Data.ByteString (ByteString)
 import           Data.Int (Int64)
 import           Data.List (intersperse)
 import           Data.Monoid (mconcat)
-import           Database.PostgreSQL.Simple.Compat ( (<>) )
+import           Database.PostgreSQL.Simple.Compat ( (<>), toByteString )
 import           Database.PostgreSQL.Simple.FromField (ResultError(..))
 import           Database.PostgreSQL.Simple.FromRow (FromRow(..))
 import           Database.PostgreSQL.Simple.Ok
@@ -173,9 +179,9 @@ formatMany conn q@(Query template) qs = do
   case parseTemplate template of
     Just (before, qbits, after) -> do
       bs <- mapM (buildQuery conn q qbits . toRow) qs
-      return . toByteString . mconcat $ fromByteString before :
-                                        intersperse (fromChar ',') bs ++
-                                        [fromByteString after]
+      return . toByteString . mconcat $ byteString before :
+                                        intersperse (char8 ',') bs ++
+                                        [byteString after]
     Nothing -> fmtError "syntax error in multi-row template" q []
 
 -- Split the input string into three pieces, @before@, @qbits@, and @after@,
@@ -280,7 +286,7 @@ buildQuery conn q template xs =
     zipParams (split template) <$> mapM (buildAction conn q xs) xs
   where split s =
             let (h,t) = B.break (=='?') s
-            in fromByteString h
+            in byteString h
                : if B.null t
                  then []
                  else split (B.tail t)
@@ -329,7 +335,7 @@ execute conn template qs = do
 --        SET sometable.y = upd.y
 --       FROM (VALUES (?,?)) as upd(x,y)
 --      WHERE sometable.x = upd.x
---  |] [(1, \"hello\"),(2, \"world\")
+--  |] [(1, \"hello\"),(2, \"world\")]
 -- @
 
 executeMany :: (ToRow q) => Connection -> Query -> [q] -> IO Int64
@@ -351,10 +357,13 @@ executeMany conn q qs = do
 --
 -- Throws 'FormatError' if the query could not be formatted correctly.
 returning :: (ToRow q, FromRow r) => Connection -> Query -> [q] -> IO [r]
-returning _ _ [] = return []
-returning conn q qs = do
+returning = returningWith fromRow
+
+returningWith :: (ToRow q) => RowParser r -> Connection -> Query -> [q] -> IO [r]
+returningWith _ _ _ [] = return []
+returningWith parser conn q qs = do
   result <- exec conn =<< formatMany conn q qs
-  finishQuery conn q result
+  finishQueryWith parser conn q result
 
 -- | Perform a @SELECT@ or other SQL query that is expected to return
 -- results. All results are retrieved and converted before this
@@ -429,6 +438,17 @@ fold            :: ( FromRow row, ToRow params )
                 -> IO a
 fold = foldWithOptions defaultFoldOptions
 
+-- | A version of 'fold' taking a parser as an argument
+foldWith        :: ( ToRow params )
+                => RowParser row
+                -> Connection
+                -> Query
+                -> params
+                -> a
+                -> (a -> row -> IO a)
+                -> IO a
+foldWith = foldWithOptionsAndParser defaultFoldOptions
+
 -- | Number of rows to fetch at a time.   'Automatic' currently defaults
 --   to 256 rows,  although it might be nice to make this more intelligent
 --   based on e.g. the average size of the rows.
@@ -463,9 +483,21 @@ foldWithOptions :: ( FromRow row, ToRow params )
                 -> a
                 -> (a -> row -> IO a)
                 -> IO a
-foldWithOptions opts conn template qs a f = do
+foldWithOptions opts = foldWithOptionsAndParser opts fromRow
+
+-- | A version of 'foldWithOptions' taking a parser as an argument
+foldWithOptionsAndParser :: (ToRow params)
+                         => FoldOptions
+                         -> RowParser row
+                         -> Connection
+                         -> Query
+                         -> params
+                         -> a
+                         -> (a -> row -> IO a)
+                         -> IO a
+foldWithOptionsAndParser opts parser conn template qs a f = do
     q <- formatQuery conn template qs
-    doFold opts conn template (Query q) a f
+    doFold opts parser conn template (Query q) a f
 
 -- | A version of 'fold' that does not perform query substitution.
 fold_ :: (FromRow r) =>
@@ -476,6 +508,14 @@ fold_ :: (FromRow r) =>
       -> IO a
 fold_ = foldWithOptions_ defaultFoldOptions
 
+-- | A version of 'fold_' taking a parser as an argument
+foldWith_ :: RowParser r
+          -> Connection
+          -> Query
+          -> a
+          -> (a -> r -> IO a)
+          -> IO a
+foldWith_ = foldWithOptionsAndParser_ defaultFoldOptions
 
 foldWithOptions_ :: (FromRow r) =>
                     FoldOptions
@@ -484,18 +524,27 @@ foldWithOptions_ :: (FromRow r) =>
                  -> a                 -- ^ Initial state for result consumer.
                  -> (a -> r -> IO a)  -- ^ Result consumer.
                  -> IO a
-foldWithOptions_ opts conn query a f = doFold opts conn query query a f
+foldWithOptions_ opts conn query a f = doFold opts fromRow conn query query a f
 
+-- | A version of 'foldWithOptions_' taking a parser as an argument
+foldWithOptionsAndParser_ :: FoldOptions
+                          -> RowParser r
+                          -> Connection
+                          -> Query             -- ^ Query.
+                          -> a                 -- ^ Initial state for result consumer.
+                          -> (a -> r -> IO a)  -- ^ Result consumer.
+                          -> IO a
+foldWithOptionsAndParser_ opts parser conn query a f = doFold opts parser conn query query a f
 
-doFold :: ( FromRow row )
-       => FoldOptions
+doFold :: FoldOptions
+       -> RowParser row
        -> Connection
        -> Query
        -> Query
        -> a
        -> (a -> row -> IO a)
        -> IO a
-doFold FoldOptions{..} conn _template q a0 f = do
+doFold FoldOptions{..} parser conn _template q a0 f = do
     stat <- withConnection conn PQ.transactionStatus
     case stat of
       PQ.TransIdle    -> withTransactionMode transactionMode conn go
@@ -518,22 +567,33 @@ doFold FoldOptions{..} conn _template q a0 f = do
         _ <- execute_ conn $ mconcat
                  [ "DECLARE ", name, " NO SCROLL CURSOR FOR ", q ]
         return name
-    fetch (Query name) = query_ conn $
-        Query (toByteString (fromByteString "FETCH FORWARD "
-                             <> integral chunkSize
-                             <> fromByteString " FROM "
-                             <> fromByteString name
-                            ))
     close name =
         (execute_ conn ("CLOSE " <> name) >> return ()) `E.catch` \ex ->
             -- Don't throw exception if CLOSE failed because the transaction is
             -- aborted.  Otherwise, it will throw away the original error.
-            if isFailedTransactionError ex then return () else throwIO ex
+            unless (isFailedTransactionError ex) $ throwIO ex
 
-    go = bracket declare close $ \name ->
-         let loop a = do
-                 rs <- fetch name
-                 if null rs then return a else foldM f a rs >>= loop
+    go = bracket declare close $ \(Query name) ->
+         let q = toByteString (byteString "FETCH FORWARD "
+                               <> intDec chunkSize
+                               <> byteString " FROM "
+                               <> byteString name
+                              )
+             loop a = do
+                 result <- exec conn q
+                 status <- PQ.resultStatus result
+                 case status of
+                     PQ.TuplesOk -> do
+                         nrows <- PQ.ntuples result
+                         ncols <- PQ.nfields result
+                         if nrows > 0
+                         then do
+                             let inner a row = do
+                                   x <- getRowWith parser row ncols conn result
+                                   f a x
+                             foldM' inner a 0 (nrows - 1) >>= loop
+                         else return a
+                     _   -> throwResultError "fold" result status
           in loop a0
 
 -- FIXME: choose the Automatic chunkSize more intelligently
@@ -553,18 +613,37 @@ forEach :: (ToRow q, FromRow r) =>
         -> q                    -- ^ Query parameters.
         -> (r -> IO ())         -- ^ Result consumer.
         -> IO ()
-forEach conn template qs = fold conn template qs () . const
+forEach = forEachWith fromRow
 {-# INLINE forEach #-}
 
+-- | A version of 'forEach' taking a parser as an argument
+forEachWith :: ( ToRow q )
+            => RowParser r
+            -> Connection
+            -> Query
+            -> q
+            -> (r -> IO ())
+            -> IO ()
+forEachWith parser conn template qs = foldWith parser conn template qs () . const
+{-# INLINE forEachWith #-}
+
 -- | A version of 'forEach' that does not perform query substitution.
 forEach_ :: (FromRow r) =>
             Connection
          -> Query                -- ^ Query template.
          -> (r -> IO ())         -- ^ Result consumer.
          -> IO ()
-forEach_ conn template = fold_ conn template () . const
+forEach_ = forEachWith_ fromRow
 {-# INLINE forEach_ #-}
 
+forEachWith_ :: RowParser r
+             -> Connection
+             -> Query
+             -> (r -> IO ())
+             -> IO ()
+forEachWith_ parser conn template = foldWith_ parser conn template () . const
+{-# INLINE forEachWith_ #-}
+
 forM' :: (Ord n, Num n) => n -> n -> (n -> IO a) -> IO [a]
 forM' lo hi m = loop hi []
   where
@@ -573,9 +652,17 @@ forM' lo hi m = loop hi []
       | otherwise = do
            a <- m n
            loop (n-1) (a:as)
+{-# INLINE forM' #-}
 
-finishQuery :: FromRow r => Connection -> Query -> PQ.Result -> IO [r]
-finishQuery = finishQueryWith fromRow
+foldM' :: (Ord n, Num n) => (a -> n -> IO a) -> a -> n -> n -> IO a
+foldM' f a lo hi = loop a lo
+  where
+    loop a !n
+      | n > hi = return a
+      | otherwise = do
+           a' <- f a n
+           loop a' (n+1)
+{-# INLINE foldM' #-}
 
 finishQueryWith :: RowParser r -> Connection -> Query -> PQ.Result -> IO [r]
 finishQueryWith parser conn q result = do
@@ -583,33 +670,13 @@ finishQueryWith parser conn q result = do
   case status of
     PQ.EmptyQuery ->
         throwIO $ QueryError "query: Empty query" q
-    PQ.CommandOk -> do
+    PQ.CommandOk ->
         throwIO $ QueryError "query resulted in a command response" q
     PQ.TuplesOk -> do
-        let unCol (PQ.Col x) = fromIntegral x :: Int
         nrows <- PQ.ntuples result
         ncols <- PQ.nfields result
-        forM' 0 (nrows-1) $ \row -> do
-           let rw = Row row result
-           okvc <- runConversion (runStateT (runReaderT (unRP parser) rw) 0) conn
-           case okvc of
-             Ok (val,col) | col == ncols -> return val
-                          | otherwise -> do
-                              vals <- forM' 0 (ncols-1) $ \c -> do
-                                  tinfo <- getTypeInfo conn =<< PQ.ftype result c
-                                  v <- PQ.getvalue result row c
-                                  return ( tinfo
-                                         , fmap ellipsis v       )
-                              throw (ConversionFailed
-                               (show (unCol ncols) ++ " values: " ++ show vals)
-                               Nothing
-                               ""
-                               (show (unCol col) ++ " slots in target type")
-                               "mismatch between number of columns to \
-                               \convert and number in target type")
-             Errors []  -> throwIO $ ConversionFailed "" Nothing "" "" "unknown error"
-             Errors [x] -> throwIO x
-             Errors xs  -> throwIO $ ManyErrors xs
+        forM' 0 (nrows-1) $ \row ->
+            getRowWith parser row ncols conn result
     PQ.CopyOut ->
         throwIO $ QueryError "query: COPY TO is not supported" q
     PQ.CopyIn ->
@@ -618,6 +685,30 @@ finishQueryWith parser conn q result = do
     PQ.NonfatalError -> throwResultError "query" result status
     PQ.FatalError    -> throwResultError "query" result status
 
+getRowWith :: RowParser r -> PQ.Row -> PQ.Column -> Connection -> PQ.Result -> IO r
+getRowWith parser row ncols conn result = do
+  let rw = Row row result
+  let unCol (PQ.Col x) = fromIntegral x :: Int
+  okvc <- runConversion (runStateT (runReaderT (unRP parser) rw) 0) conn
+  case okvc of
+    Ok (val,col) | col == ncols -> return val
+                 | otherwise -> do
+                     vals <- forM' 0 (ncols-1) $ \c -> do
+                         tinfo <- getTypeInfo conn =<< PQ.ftype result c
+                         v <- PQ.getvalue result row c
+                         return ( tinfo
+                                , fmap ellipsis v       )
+                     throw (ConversionFailed
+                      (show (unCol ncols) ++ " values: " ++ show vals)
+                      Nothing
+                      ""
+                      (show (unCol col) ++ " slots in target type")
+                      "mismatch between number of columns to \
+                      \convert and number in target type")
+    Errors []  -> throwIO $ ConversionFailed "" Nothing "" "" "unknown error"
+    Errors [x] -> throwIO x
+    Errors xs  -> throwIO $ ManyErrors xs
+
 ellipsis :: ByteString -> ByteString
 ellipsis bs
     | B.length bs > 15 = B.take 10 bs `B.append` "[...]"
