@@ -28,10 +28,10 @@ conversion routine,  which sacrifices some accuracy for speed.   If you
 need accuracy,  consider first converting data to a 'Scientific' or 'Rational'
 type,  and then converting to a floating-point type.   If you are defining
 your own 'Database.PostgreSQL.Simple.FromRow.FromRow' instances,  this can be
-acheived simply by
+achieved simply by
 @'fromRational' '<$>' 'Database.PostgreSQL.Simple.FromRow.field'@,  although
-this idiom is additionally compatible with PostgreSQL's @numeric@ type.
-If this is unacceptable,  you may find
+this idiom is additionally compatible with PostgreSQL's @int8@ and @numeric@
+types.  If this is unacceptable,  you may find
 'Database.PostgreSQL.Simple.FromRow.fieldWith' useful.
 
 Also note that while converting to a 'Double' through the 'Scientific' type
@@ -112,10 +112,11 @@ module Database.PostgreSQL.Simple.FromField
 
 #include "MachDeps.h"
 
-import           Control.Applicative ( (<|>), (<$>), pure, (*>) )
+import           Control.Applicative ( (<|>), (<$>), pure, (*>), (<*) )
 import           Control.Concurrent.MVar (MVar, newMVar)
 import           Control.Exception (Exception)
 import qualified Data.Aeson as JSON
+import qualified Data.Aeson.Parser as JSON (value')
 import           Data.Attoparsec.ByteString.Char8 hiding (Result)
 import           Data.ByteString (ByteString)
 import qualified Data.ByteString.Char8 as B
@@ -145,8 +146,8 @@ import qualified Data.Text.Encoding as ST
 import qualified Data.Text.Lazy as LT
 import           Data.CaseInsensitive (CI)
 import qualified Data.CaseInsensitive as CI
-import           Data.UUID   (UUID)
-import qualified Data.UUID as UUID
+import           Data.UUID.Types   (UUID)
+import qualified Data.UUID.Types as UUID
 import           Data.Scientific (Scientific)
 import           GHC.Real (infinity, notANumber)
 
@@ -217,7 +218,7 @@ typename field = typname <$> typeInfo field
 
 typeInfo :: Field -> Conversion TypeInfo
 typeInfo Field{..} = Conversion $ \conn -> do
-                       Ok <$> (getTypeInfo conn =<< PQ.ftype result column)
+                       Ok <$> (getTypeInfo conn typeOid)
 
 typeInfoByOid :: PQ.Oid -> Conversion TypeInfo
 typeInfoByOid oid = Conversion $ \conn -> do
@@ -334,15 +335,15 @@ instance FromField Double where
     fromField = atto ok pg_double
       where ok = $(mkCompats [TI.float4,TI.float8,TI.int2,TI.int4])
 
--- | int2, int4, float4, float8, numeric
+-- | int2, int4, int8, float4, float8, numeric
 instance FromField (Ratio Integer) where
     fromField = atto ok pg_rational
-      where ok = $(mkCompats [TI.float4,TI.float8,TI.int2,TI.int4,TI.numeric])
+      where ok = $(mkCompats [TI.float4,TI.float8,TI.int2,TI.int4,TI.int8,TI.numeric])
 
--- | int2, int4, float4, float8, numeric
+-- | int2, int4, int8, float4, float8, numeric
 instance FromField Scientific where
      fromField = atto ok rational
-      where ok = $(mkCompats [TI.float4,TI.float8,TI.int2,TI.int4,TI.numeric])
+      where ok = $(mkCompats [TI.float4,TI.float8,TI.int2,TI.int4,TI.int8,TI.numeric])
 
 unBinary :: Binary t -> t
 unBinary (Binary x) = x
@@ -545,13 +546,7 @@ instance FromField JSON.Value where
       else case mbs of
              Nothing -> returnError UnexpectedNull f ""
              Just bs ->
-#if MIN_VERSION_aeson(0,6,3)
-                 case JSON.eitherDecodeStrict' bs of
-#elif MIN_VERSION_bytestring(0,10,0)
-                 case JSON.eitherDecode' $ LB.fromStrict bs of
-#else
-                 case JSON.eitherDecode' $ LB.fromChunks [bs] of
-#endif
+                 case parseOnly (JSON.value' <* endOfInput) bs of
                    Left  err -> returnError ConversionFailed f err
                    Right val -> pure val
 
