@@ -20,8 +20,6 @@
 
 module Database.PostgreSQL.Simple.Internal where
 
-import           Blaze.ByteString.Builder
-                   ( Builder, fromByteString, toByteString )
 import           Control.Applicative
 import           Control.Exception
 import           Control.Concurrent.MVar
@@ -29,6 +27,7 @@ import           Control.Monad(MonadPlus(..))
 import           Data.ByteString(ByteString)
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as B8
+import           Data.ByteString.Builder ( Builder, byteString )
 import           Data.Char (ord)
 import           Data.Int (Int64)
 import qualified Data.IntMap as IntMap
@@ -43,6 +42,7 @@ import           Data.Word
 import           Database.PostgreSQL.LibPQ(Oid(..))
 import qualified Database.PostgreSQL.LibPQ as PQ
 import           Database.PostgreSQL.LibPQ(ExecStatus(..))
+import           Database.PostgreSQL.Simple.Compat ( toByteString )
 import           Database.PostgreSQL.Simple.Ok
 import           Database.PostgreSQL.Simple.ToField (Action(..), inQuotes)
 import           Database.PostgreSQL.Simple.Types (Query(..))
@@ -75,7 +75,10 @@ data Connection = Connection {
      connectionHandle  :: {-# UNPACK #-} !(MVar PQ.Connection)
    , connectionObjects :: {-# UNPACK #-} !(MVar TypeInfoCache)
    , connectionTempNameCounter :: {-# UNPACK #-} !(IORef Int64)
-   }
+   } deriving (Typeable)
+
+instance Eq Connection where
+   x == y = connectionHandle x == connectionHandle y
 
 data SqlError = SqlError {
      sqlState       :: ByteString
@@ -152,8 +155,11 @@ connect = connectPostgreSQL . postgreSQLConnectionString
 
 -- | Attempt to make a connection based on a libpq connection string.
 --   See <http://www.postgresql.org/docs/9.3/static/libpq-connect.html#LIBPQ-CONNSTRING>
---   for more information.  Here is an example with some
---   of the most commonly used parameters:
+--   for more information.  Also note that environment variables also affect
+--   parameters not provided, parameters provided as the empty string, and a
+--   few other things; see <http://www.postgresql.org/docs/9.3/static/libpq-envars.html>
+--   for details.  Here is an example with some of the most commonly used
+--   parameters:
 --
 -- > host='db.somedomain.com' port=5432 ...
 --
@@ -206,9 +212,9 @@ connect = connectPostgreSQL . postgreSQLConnectionString
 --   SSL/TLS will typically "just work" if your postgresql server supports or
 --   requires it.  However,  note that libpq is trivially vulnerable to a MITM
 --   attack without setting additional SSL parameters in the connection string.
---   In particular,  @sslmode@ needs to set be @require@, @verify-ca@, or
---   @verify-full@ to perform certificate validation.   When @sslmode@ is
---   @require@,  then you will also need to have a @sslrootcert@ file,
+--   In particular,  @sslmode@ needs to be set to @require@, @verify-ca@, or
+--   @verify-full@ in order to perform certificate validation.  When @sslmode@
+--   is @require@,  then you will also need to specify a @sslrootcert@ file,
 --   otherwise no validation of the server's identity will be performed.
 --   Client authentication via certificates is also possible via the
 --   @sslcert@ and @sslkey@ parameters.
@@ -225,8 +231,8 @@ connectPostgreSQL connstr = do
           let wconn = Connection{..}
           version <- PQ.serverVersion conn
           let settings
-                | version < 80200 = "SET datestyle TO ISO"
-                | otherwise       = "SET standard_conforming_strings TO on;SET datestyle TO ISO"
+                | version < 80200 = "SET datestyle TO ISO;SET client_encoding TO UTF8"
+                | otherwise       = "SET datestyle TO ISO;SET client_encoding TO UTF8;SET standard_conforming_strings TO on"
           _ <- execute_ wconn settings
           return wconn
       _ -> do
@@ -538,7 +544,7 @@ fmtErrorBs q xs msg = fmtError (T.unpack $ TE.decodeUtf8 msg) q xs
 
 -- | Quote bytestring or throw 'FormatError'
 quote :: Query -> [Action] -> Either ByteString ByteString -> Builder
-quote q xs = either (fmtErrorBs q xs) (inQuotes . fromByteString)
+quote q xs = either (fmtErrorBs q xs) (inQuotes . byteString)
 
 buildAction :: Connection        -- ^ Connection for string escaping
             -> Query             -- ^ Query for message error
@@ -549,7 +555,7 @@ buildAction _ _ _     (Plain  b)            = pure b
 buildAction conn q xs (Escape s)            = quote q xs <$> escapeStringConn conn s
 buildAction conn q xs (EscapeByteA s)       = quote q xs <$> escapeByteaConn conn s
 buildAction conn q xs (EscapeIdentifier s) =
-    either (fmtErrorBs q xs) fromByteString <$> escapeIdentifier conn s
+    either (fmtErrorBs q xs) byteString <$> escapeIdentifier conn s
 buildAction conn q xs (Many  ys)           =
     mconcat <$> mapM (buildAction conn q xs) ys
 
